==> omnibuild-0.1.0.tar.gz: lib/src/parse.dart <==
// Copyright (c) 2015, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

/// Parse YAMl data into an OmnibuildSpecFile instance.
_parseYaml(OmnibuildSpecFile spec, String yamlData) {
  final yaml = loadYaml(yamlData);

  // Load copyright, library and description.
  spec.copyright = yaml.containsKey('copyright') ? yaml['copyright'] : '';
  spec.library = yaml.containsKey('library') ? yaml['library'] : '';
  spec.description = yaml.containsKey('description') ? yaml['description'] : '';

  // Loop through messages.
  if (yaml.containsKey('messages')) {
    yaml['messages'].forEach((YamlMap message) {
      spec.messages.add(_parseMessageYaml(message, spec.package, spec.version));
    });
  }
}

/// Parse YAML message data.
OmnibuildMessage _parseMessageYaml(YamlMap data, String package, int version) {
  // Message name (non-optional)
  final name = data['name'];

  // Get messate type as OmnibusMessageType.
  // Enforces lower-case convention on purpose.
  const messageTypes = const {
    'event': OmnibusMessageType.event,
    'request': OmnibusMessageType.request,
    'response': OmnibusMessageType.response
  };
  final type = messageTypes[(data.containsKey('type') &&
      messageTypes.containsKey(data['type']) ? data['type'] : 'event')];

  // Message documentation
  final documentation = data.containsKey('note') ? data['note'] : '';

  // Message properties
  var properties = new List<OmnibuildProperty>();
  if (data.containsKey('properties')) {
    data['properties'].forEach((YamlMap property) {
      properties.add(_parsePropertyYaml(property));
    });
  }

  return new OmnibuildMessage(
      '$package.v$version.$name', name, documentation, type, properties);
}

OmnibuildProperty _parsePropertyYaml(YamlMap data) {
  // name and type are non-optional
  final String name = data['name'];
  String type = data['type'];

  // Enumeration types
  bool enumeration = false;
  if (type.startsWith('enum ')) {
    enumeration = true;
    type = type.substring(5);
  }

  // Message documentation
  final String documentation = data.containsKey('note') ? data['note'] : '';

  // Pass property as named parameter in message builder.
  // Enforces lower-case convention on purpose.
  // Defaults to positional parameter (namedParameter = false).
  final bool namedParameter =
      data.containsKey('pass') ? data['pass'] == 'named parameter' : false;

  // Stringified default value. Only works for dart primitive types that can be
  // expressed as string in YAML (boolean, integer, list, map).
  final String defaultValue = _resolveDefaultValueCode(type,
      data.containsKey('dflt') ? data['dflt'].toString() : '', namedParameter);

  return new OmnibuildProperty(
      name, documentation, type, enumeration, namedParameter, defaultValue);
}

/// Generate default value for OmnibuildProperty.
String _resolveDefaultValueCode(String type, String dflt, bool namedParameter) {
  if (type == 'String') {
    if (dflt.isNotEmpty) {
      // Single line strings
      if (!dflt.contains('\n')) {
        if (!dflt.contains("'")) {
          // Use single quotes if the string contains no single quotes.
          return "'$dflt'";
        } else if (!dflt.contains('"')) {
          // Use double quotes if the string contains no double quotes.
          return '"$dflt"';
        } else {
          // Use single quotes and escape all single quotes in the string tha
          // are not already escaped (tricky).
          return "'${_escapeSingleQuotes(dflt)}'";
        }
      } else {
        if (!dflt.contains("'''")) {
          // Use triple quotes.
          return "'''$dflt'''";
        } else {
          // Escape all single quotes in the string and use triple quotes.
          return "'''${_escapeSingleQuotes(dflt)}'''";
        }
      }
    } else {
      return namedParameter ? "''" : '';
    }
  } else if (dflt.isEmpty) {
    // Generate standard dflt value if necessary.
    if (namedParameter) {
      if (type.startsWith('List<')) {
        return 'const []';
      } else if (type.startsWith('Map<')) {
        return 'const {}';
      } else {
        return 'null';
      }
    } else {
      return '';
    }
  } else {
    return dflt;
  }
}

String _escapeSingleQuotes(String src) {
  return src.replaceAll("'", "\\'");
}

==> omnibuild-0.2.0.tar.gz: lib/src/parse.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

/// Parse YAMl data into an OmnibuildSpecFile instance.
void _parseYaml(OmnibuildSpecFile spec, String yamlString) {
  final yaml = loadYaml(yamlString);

  // Load copyright, library and description.
  spec.copyright = yaml.containsKey('copyright') ? yaml['copyright'] : '';
  spec.library = yaml.containsKey('library') ? yaml['library'] : '';
  spec.description = yaml.containsKey('description') ? yaml['description'] : '';

  // Loop through messages.
  if (yaml.containsKey('messages')) {
    yaml['messages'].forEach((YamlMap message) {
      spec.messages.add(_parseMessageYaml(message, spec.package, spec.version));
    });
  }
}

/// Parse YAML message data.
OmnibuildMessage _parseMessageYaml(YamlMap data, String package, int version) {
  // Message name (non-optional)
  final name = data['name'];

  // Get messate type as OmnibusMessageType.
  // Enforces lower-case convention on purpose.
  const messageTypes = const {
    'event': OmnibusMessageType.event,
    'request': OmnibusMessageType.request,
    'response': OmnibusMessageType.response
  };
  final type = messageTypes[(data.containsKey('type') &&
      messageTypes.containsKey(data['type']) ? data['type'] : 'event')];

  // Message documentation
  final documentation = data.containsKey('note') ? data['note'] : '';

  // Message properties
  var properties = new List<OmnibuildProperty>();
  if (data.containsKey('properties')) {
    data['properties'].forEach((YamlMap property) {
      properties.add(_parsePropertyYaml(property));
    });
  }

  return new OmnibuildMessage(
      '$package.v$version.$name', name, documentation, type, properties);
}

OmnibuildProperty _parsePropertyYaml(YamlMap data) {
  // name and type are non-optional
  final String name = data['name'];
  String type = data['type'];

  // Impure maps are not supported.
  if (type.startsWith('Map<') && !_isJsonType(type)) {
    throw new ArgumentError.value(
        data, 'data', 'Non-primitive Maps are not supported');
  }

  // Enumeration types
  bool enumeration = false;
  if (type.startsWith('enum ')) {
    enumeration = true;
    type = type.substring(5);
  }

  // JSON type
  final String jsonType =
      data.containsKey('json') ? data['json'] : enumeration ? 'int' : type;

  // If this is an enumeration, the jsonType must be an integer.
  if (enumeration) {
    if (jsonType != 'int') {
      throw new ArgumentError.value(
          data, 'data', 'The JSON type for enums should always be an integer');
    }
  } else if (_isJsonType(type) && jsonType != type) {
    // The jsonType can only be different from the actual type if the actual
    // type is not already pure JSON data.
    throw new ArgumentError.value(data, 'data',
        'Data contains different JSON type while the actual Dart type is already JSON compatible');
  }

  // Message documentation
  final String documentation = data.containsKey('note') ? data['note'] : '';

  // Pass property as named parameter in message builder.
  // Enforces lower-case convention on purpose.
  // Defaults to positional parameter (namedParameter = false).
  final bool namedParameter =
      data.containsKey('pass') ? data['pass'] == 'named parameter' : false;

  // Stringified default value. Only works for dart primitive types that can be
  // expressed as string in YAML (boolean, integer, list, map).
  final String defaultValue = _resolveDefaultValueCode(type,
      data.containsKey('dflt') ? data['dflt'].toString() : '', namedParameter);

  return new OmnibuildProperty(name, documentation, type, jsonType, enumeration,
      namedParameter, defaultValue);
}

/// Generate default value for OmnibuildProperty.
String _resolveDefaultValueCode(String type, String dflt, bool namedParameter) {
  if (type == 'String') {
    if (dflt.isNotEmpty) {
      // Single line strings
      if (!dflt.contains('\n')) {
        if (!dflt.contains("'")) {
          // Use single quotes if the string contains no single quotes.
          return "'$dflt'";
        } else if (!dflt.contains('"')) {
          // Use double quotes if the string contains no double quotes.
          return '"$dflt"';
        } else {
          // Use single quotes and escape all single quotes in the string tha
          // are not already escaped (tricky).
          return "'${_escapeSingleQuotes(dflt)}'";
        }
      } else {
        if (!dflt.contains("'''")) {
          // Use triple quotes.
          return "'''$dflt'''";
        } else {
          // Escape all single quotes in the string and use triple quotes.
          return "'''${_escapeSingleQuotes(dflt)}'''";
        }
      }
    } else {
      return namedParameter ? "''" : '';
    }
  } else if (dflt.isEmpty) {
    // Generate standard dflt value if necessary.
    if (namedParameter) {
      if (type.startsWith('List<')) {
        return 'const []';
      } else if (type.startsWith('Map<')) {
        return 'const {}';
      } else {
        return 'null';
      }
    } else {
      return '';
    }
  } else {
    return dflt;
  }
}

String _escapeSingleQuotes(String src) {
  return src.replaceAll("'", "\\'");
}
